"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useCodeMirror = void 0;

var _react = _interopRequireDefault(require("react"));

var _view = require("@codemirror/view");

var _state = require("@codemirror/state");

var _createEditorView = require("@ui-schema/kit-codemirror/createEditorView");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var useCodeMirror = function useCodeMirror(onChange) {
  var _effects;

  var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var extensions = arguments.length > 2 ? arguments[2] : undefined;
  var effects = arguments.length > 3 ? arguments[3] : undefined;
  var containerRef = arguments.length > 4 ? arguments[4] : undefined;
  var onExternalChange = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : _createEditorView.replaceWholeDoc;
  var onViewLifecycle = arguments.length > 6 ? arguments[6] : undefined;

  var lastValueRef = _react["default"].useRef(value);

  var onChangeRef = _react["default"].useRef(undefined);

  var _React$useState = _react["default"].useState(undefined),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      editor = _React$useState2[0],
      setEditor = _React$useState2[1];

  onChangeRef.current = onChange;

  var readOnlyCompartment = _react["default"].useRef(new _state.Compartment());

  _react["default"].useLayoutEffect(function () {
    var editor = (0, _createEditorView.createEditorView)(lastValueRef, [].concat(_toConsumableArray(extensions || []), [readOnlyCompartment.current.of(_view.EditorView.editable.of(Boolean(onChangeRef.current)))]), onChangeRef);

    if (containerRef) {
      var _containerRef$current;

      (_containerRef$current = containerRef.current) === null || _containerRef$current === void 0 ? void 0 : _containerRef$current.append(editor.dom);
    }

    setEditor(editor);
    onViewLifecycle === null || onViewLifecycle === void 0 ? void 0 : onViewLifecycle(editor);
    return function () {
      onViewLifecycle === null || onViewLifecycle === void 0 ? void 0 : onViewLifecycle(editor, true);
      editor === null || editor === void 0 ? void 0 : editor.destroy();
      setEditor(undefined);
    };
  }, [containerRef, extensions, onViewLifecycle]);

  _react["default"].useEffect(function () {
    if (!editor) return;
    editor.dispatch({
      effects: readOnlyCompartment.current.reconfigure(_view.EditorView.editable.of(Boolean(onChange)))
    });
  }, [editor, onChange]);

  _react["default"].useLayoutEffect(function () {
    if (!editor) return;

    if (lastValueRef.current !== value) {
      var textA = editor.state.doc.toString();
      var textB = value;
      onExternalChange(editor, textB, textA);
      lastValueRef.current = textB;
    } else {
      lastValueRef.current = value;
    }
  }, [value, editor, onExternalChange, containerRef]);

  effects = ((_effects = effects) === null || _effects === void 0 ? void 0 : _effects.length) === 0 ? undefined : effects;

  _react["default"].useLayoutEffect(function () {
    if (!editor && effects) {
      console.error('received effects but editor is not ready', effects);
      return;
    }

    if (!effects || !editor) return;
    effects.forEach(function (effect) {
      effect(editor);
    });
  }, [effects, editor]);

  return editor;
};

exports.useCodeMirror = useCodeMirror;